<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Operador as OperadorRepository;
use ACL\Traits\TCrud;
use ACL\Util\AclUtilities;
use ACL\Util\Messages;
use Exception;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\Libs\Util\Utilities as LibUtilities;

class Operador {

    use TCrud;

    const ACTIVE_ID = 'S';
    const NOT_ACTIVE_ID = 'S';

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var OperadorRepository
     */
    private $operadorRepository;

    /**
     * @var // FileServerFilesUrl
     */
    private $fileServerFilesUrl;
    /**
     * @var Server
     */
    private $zhFileUploader;

    /**
     * Operador constructor.
     * @param $fileServerFilesUrl
     * @param Server $zhFileUploader
     * @param OperadorRepository $operadorRepository
     * @throws Exception
     */
    public function __construct($fileServerFilesUrl, Server $zhFileUploader,OperadorRepository $operadorRepository)
    {
        $this->fileServerFilesUrl = $fileServerFilesUrl;
        $this->zhFileUploader = $zhFileUploader;
        $this->operadorRepository = $operadorRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @param $params
     * @return array
     * @throws Exception
     */
    public function findAllOperador($params): array
    {
        $idoperativoFilter = $this->getIdoperativoFilter($params);
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();
        return $this->operadorRepository->findAllOperador($this->getOrganizationId(), $idoperativoFilter, $oAccCustomOrgId);
    }

    public function createOperador(array $params)
    {
        extract($params);
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $parcnegocioService = InstanceProvider::getService()->getParcnegocio();
        $connection->beginTransaction();
        try {

            if (!$CDOPERADOR) {
                $CDOPERADOR = $this->generateCdoperador();
            }
            $this->operadorSaveValidation($CDOPERADOR, $DSEMAILOPER, getNrorg(), $NEW_PASSWORD, $CONFIRM_PASSWORD);
            $parcnegocios = $this->operatorSaveValidationCpf($NRINSCRICAOPARC);

            $NRPARCNEGOCIO = $this->setParcnegocio($NRPARCNEGOCIO, $NMOPERADOR, $NRINSCRICAOPARC, $CDTIPOINSCRICAO,
                                                   $IDATIVO, $parcnegocios, $entityManager);

            $parcnegocioService->createParceria($NRPARCNEGOCIO, getNrorg(), getCurrentUserId(), 'OPERADOR');
            $entityManager->flush();

            if ($CDCLIENTE != null) {
                $parcnegocioService->createParceria($NRPARCNEGOCIO, getNrorg(), getCurrentUserId(), 'INTERLOCUTOR');
                $entityManager->flush();
            }

            $senhaMd5 = md5($NEW_PASSWORD);

            $this->operadorRepository->createOperador(
                $CDOPERADOR, $IDOPERATIVO, $IDATIVO, $NMOPERADOR, $NMACESSO, $DSEMAILOPER,
                $CDPAIS, $NRPARCNEGOCIO, $CDCLIENTE, getNrorg(), getNrorg(),null,
                getNrorg(), getCurrentUserId(), $senhaMd5, $IDOPERMASTER, null,
                $IDTPOPERADOR, $NRORGOPER);
            $entityManager->flush();

            $this->createOrganizacaooper(getNrorg(), $CDOPERADOR, getCurrentUserId());
            $entityManager->flush();

            $this->operadorRepository->updateParcnegocioInOperador($NRPARCNEGOCIO, $CDOPERADOR, getNrorg());
            $entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param array $params
     * @return array
     * @throws Exception
     */
    public function updateOperador(array $params): array
    {
        extract($params);
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $this->operadorEditValidation($CDOPERADOR, $DSEMAILOPER);
            $parcnegocios = $this->operatorEditValidationCpf($NRINSCRICAOPARC, $NRPARCNEGOCIO);

            $NRPARCNEGOCIO = $this->setParcnegocioByEdit($NRPARCNEGOCIO, $NMOPERADOR, getNrorg(), $NRINSCRICAOPARC, $CDTIPOINSCRICAO, $IDATIVO,
                                        $parcnegocios, $OLD_CPF, $OLD_NMOPERADOR, $entityManager);

            if($OLD_CDCLIENTE != $CDCLIENTE){
                $this->setChangeClient($CDCLIENTE, $NRPARCNEGOCIO, $entityManager);
            }

            $this->operadorRepository->editOperator($CDOPERADOR, $IDOPERATIVO, $IDATIVO, $NMOPERADOR, $NMACESSO,
                                                    $DSEMAILOPER, $CDPAIS, $CDCLIENTE, getNrorg(), getCurrentUserId(),
                                                    $NRPARCNEGOCIO, $IDOPERMASTER, null, $IDTPOPERADOR, $NRORGOPER);
            $entityManager->flush();
            $connection->commit();

            return array("CDOPERADOR" => $CDOPERADOR, "NRPARCNEGOCIO"  => $NRPARCNEGOCIO);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findOperadoresByPerfilAcesso($nrperfilacesso): array
    {
        return $this->operadorRepository->findOperadoresByPerfilAcesso($nrperfilacesso, $this->getOrganizationId());
    }

    /**
     * @param $nrperfilacesso
     * @return array
     * @throws Exception
     */
    public function findOperadoresForAddPerfilAcesso($nrperfilacesso): array
    {
        return $this->operadorRepository->findOperadoresForAddPerfilAcesso($nrperfilacesso, $this->getOrganizationId());
    }

    /**
     * @param $nrorgoper
     * @param false $isPrimaryOrg
     * @param null $showFullOperator
     * @param null $search
     * @return array|array[]|mixed
     * @throws \Doctrine\DBAL\Exception
     */
    public function findOrganizacaoRelacionadaByOperador($nrorgoper, &$isPrimaryOrg = false, $showFullOperator = null, $search = null): array
    {
        $filioperRepository = InstanceProvider::getRepository()->getFilioper();
        $organizacaoRepository = InstanceProvider::getRepository()->getOrganizacao();
        $oacctekmgrRepository = InstanceProvider::getRepository()->getOAccTekMgr();

        $idUtlPerfilfilial = $this->libEnvironment->getIdUtlPerfilFilial();
        $oAccTekMgr = $oacctekmgrRepository->findOneBy(array('NRORG' => getNrorg()), getNrorg());
        $oAccTek = @json_decode(LibUtilities::zeedhiDecrypt($oAccTekMgr['OACCTEK_PHP7'], LibUtilities::LOGIN_CRYPT_STRING), true);
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();

        $data = [];
        if (!empty($oAccTek['relatedOrgs'])) {
            $operator = $this->operadorRepository->findOneBy(array('CDOPERADOR' => getCurrentUserId()));
            if(!empty($operator['NRORGOPER']) && $operator['NRORGOPER'] != 1) {
                $searchValue = $operator['NRORGOPER'];
            } else {
                $searchValue = !empty($oAccCustomOrgId) ? $oAccCustomOrgId : getNrorg();
            }
            $key = array_search($searchValue, array_column($oAccTek['relatedOrgs'], 'NRORG'));
            if ($key !== false) {
                if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N') && $showFullOperator) {
                    $filterFilioper = "AND CDOPERADOR = '". $operator['CDOPERADOR'] . "'";
                    $relatedBranches = $filioperRepository->findFilioper($filterFilioper);

                    $organizacoes = $this->getOrganizacoesRelacionadas($relatedBranches, $oAccTek['relatedOrgs'], $isPrimaryOrg);
                    $data = $organizacoes;
                } else {
                    if ($oAccTek['relatedOrgs'][$key]['NRORGPRINCIPAL'] === 'S') {
                        $data = [$oAccTek['relatedOrgs'][$key]];
                    } else {
                        $data = $oAccTek['relatedOrgs'];
                    }
                }
            }
        } else {
            if (empty($oAccTek['relatedOrgIds'])) {
                $oAccTek['relatedOrgIds'] = [getNrorg()];
            }
            $filter = AclUtilities::getFilterTypeIn($oAccTek['relatedOrgIds'], 'NRORG');
            $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
            if (empty($data)) {
                $filter = AclUtilities::getFilterTypeIn([getNrorg()], 'NRORG');
                $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
            }
        }

        if ($nrorgoper && $nrorgoper != 1 && array_search($nrorgoper, array_column($data, 'NRORG')) === false) {
            return [[
                'NRORG' => $nrorgoper,
                'NMORGANIZACAOOPER' => $nrorgoper . ' - Organização não encontrada',
                'NMRAZSOCIALORG' => $nrorgoper . ' - Organização não encontrada'
            ]];
        } else if (empty($data)) {
            $filter = AclUtilities::getFilterTypeIn([getNrorg()], 'NRORG');
            $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
        }

        return $this->filterOrganizacaoRelacionadaByOperador($data, $search);
    }

    /**
     * @param $data
     * @param $search
     * @return array
     */
    private function filterOrganizacaoRelacionadaByOperador($data, $search): array
    {
        $key = array_search($search, array_column($data, 'NRORG'));
        if ($key) {
            return [$data[$key]];
        }
        return $data;
    }

    private function getOrganizationIdFromOAccCustom() {
        $oAccCustom = array();
        try {
            $fileDir = '../../../login/backend/config/oAccCustom.tek';
            if (file_exists($fileDir)) {
                $fileContents  = file_get_contents($fileDir);
                $fileDecrypted = LibUtilities::zeedhiDecrypt($fileContents, LibUtilities::LOGIN_CRYPT_STRING);
                $fileDecoded   = @json_decode($fileDecrypted, true);
                if(isset($fileDecoded['organizationId']) && is_numeric($fileDecoded['organizationId'])) {
                    $oAccCustom = $fileDecoded;
                }
            }
        } catch(Exception $e) {}

        return !empty($oAccCustom) ? $oAccCustom['organizationId'] : null;
    }

    /**
     * @param $params
     * @return string
     */
    private function getIdoperativoFilter($params): string
    {
        $idoperativoFilter = '';
        if (isset($params['dynamic_filter'])) {
            $dynamicFilter = json_decode(base64_decode($params['dynamic_filter']), true);
            if (isset($dynamicFilter['IDATIVO']) && $dynamicFilter['IDATIVO'][0]['value'] === 'S') {
                $idoperativoFilter = "AND IDOPERATIVO = 'A'";
            } else {
                $idoperativoFilter = "AND IDOPERATIVO = 'I'";
            }
        }
        return $idoperativoFilter;
    }

    /**
     * @return mixed|null
     * @throws Exception
     */
    private function getOrganizationId()
    {
        return InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
    }

    /**
     * @param $relatedBranches
     * @param $orgs
     * @param $isPrimaryOrg
     * @return array|mixed
     */
    private function getOrganizacoesRelacionadas($relatedBranches, $orgs, &$isPrimaryOrg): array
    {
        $cnpjEqualArray = [];
        foreach($relatedBranches as $branch){
            $isCnpEqual = array_filter($orgs, function($o) use ($branch, &$isPrimaryOrg){
                if($o['NRINSJUREMPR'] == $branch['NRINSJURFILI']) {

                    if($o['NRORGPRINCIPAL'] == 'P'){
                        $isPrimaryOrg = true;
                    }
                    return $o;
                } else {
                    return false;
                }
            });
            if($isCnpEqual){
                $cnpjEqualArray[] = array_values($isCnpEqual)[0];
            }
        }

        return $isPrimaryOrg ? $orgs : $cnpjEqualArray;
    }

    /**
     * @param $cdoperador
     * @param $email
     * @param $organizationId
     * @param $password
     * @param $confirmPassword
     * @throws Exception
     */
    private function operadorSaveValidation($cdoperador, $email, $organizationId, $password, $confirmPassword)
    {
        $msgError = '';
        $operador = $this->operadorRepository->findOperadorByCdoperador($cdoperador, getNrorg());

        if(!empty($operador)) {
            if($operador[0]['NRORG'] == $organizationId) {
                $msgError .= Messages::ERR_EXISTS_OPERATOR_CODE;
            } else {
                $msgError .= Messages::ERR_EXISTS_OPERATOR_CODE_DIFFERENT_ORGANIZATION;
            }
        }

        $organizationId = getNrorg();
        if($this->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$email}' AND NRORG = {$organizationId}") > 0){
            $msgError .= Messages::ERR_EXISTS_EMAIL;
        }

        if($msgError){
            throw new \Exception($msgError);
        }

        $this->passwordValidate($password, $confirmPassword);
    }

    /**
     * @param $cdoperador
     * @param $dsemailoper
     * @throws Exception
     */
    public function operadorEditValidation($cdoperador, $dsemailoper)
    {
        $organizationId = getNrorg();
        $msgError = '';

        if($this->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$dsemailoper}' AND NRORG = {$organizationId} AND CDOPERADOR != '{$cdoperador}'") > 0){
            $msgError .= Messages::ERR_EXISTS_EMAIL;
        }

        if($msgError){
            throw new \Exception($msgError);
        }
    }

    /**
     * @param $newPassword
     * @param $confirmPassword
     * @param null $password
     * @throws Exception
     */
    private function passwordValidate($newPassword, $confirmPassword, $password = null)
    {
        if($newPassword != $confirmPassword){
            throw new \Exception(Messages::ERR_PASSWORD_NOT_MATCH);
        }

        if($password && $password === $newPassword){
            throw new \Exception(Messages::ERR_NEW_PASSWORD_EQUAL_OLD);
        }
    }

    /**
     * @param $cpf
     * @return array|array[]
     * @throws \Doctrine\DBAL\Exception
     */
    private function operatorSaveValidationCpf($cpf): array
    {
        if ($cpf){
            $parcnegocioRepository = InstanceProvider::getRepository()->getParcnegocio();
            $cpfIsUsed = false;

            $parceegocios = $parcnegocioRepository->findBy(array('NRINSCRICAOPARC' => $cpf), getNrorg());

            foreach ($parceegocios as $pn) {
                $operators = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), getNrorg());
                if(!empty($operators)) {
                    $cpfIsUsed = true;
                    break;
                }
            }
            if($cpfIsUsed){
                throw new \Exception(Messages::ERR_EXISTS_CPF);
            }
            return $parceegocios;
        }
        return [];
    }

    /**
     * @param $nrinscricaoparc
     * @param $nrparcnegocio
     * @return array
     * @throws \Doctrine\DBAL\Exception
     */
    public function operatorEditValidationCpf($nrinscricaoparc, $nrparcnegocio): array
    {
        $organizationId = getNrorg();
        if($nrinscricaoparc){
            $cpfIsUsed = false;
            $parcnegocios = $this->select('PARCNEGOCIO', '*', "WHERE NRINSCRICAOPARC = '{$nrinscricaoparc}' AND NRORG = {$organizationId} AND NRPARCNEGOCIO != '{$nrparcnegocio}'");

            foreach ($parcnegocios as $pn) {
                $operadores = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), $organizationId);
                if(!empty($operadores)) {
                    $cpfIsUsed = true;
                    break;
                }
            }

            if($cpfIsUsed){
                throw new \Exception(Messages::ERR_EXISTS_CPF);
            }
            return $parcnegocios;
        }
        return [];
    }

    private function createOrganizacaooper($nrorg, $cdoperador, $cdoperadorInclusao)
    {
        $organizacaooperRepository = InstanceProvider::getRepository()->getOrganizacaooper();
        $nrorganizacaooper = InstanceProvider::getNewCode()->getCodeByTableName('ORGANIZACAOOPER', $nrorg);
        $idativo = 'S';
        $organizacaooperRepository->createOrganizacaooper($nrorganizacaooper, $nrorg, $cdoperador, $nrorg, getCurrentUserId(), $idativo);
    }

    /**
     * @param $nrparcnegocio
     * @param $nmoperador
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $idativo
     * @param $parcnegocios
     * @param $entityManager
     * @return mixed
     * @throws \Doctrine\DBAL\Exception
     */
    private function setParcnegocio($nrparcnegocio, $nmoperador, $nrinscricaoparc, $cdtipoinscricao, $idativo, $parcnegocios, $entityManager)
    {
        if ($nrparcnegocio) {
            $operadores = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $nrparcnegocio), getNrorg());

            if($operadores) {
                throw new \Exception(Messages::ERR_INVALID_PARCNEGOCIO);
            }
            return $nrparcnegocio;
        } else {
            if(empty($parcnegocios)) {
                $nrparcnegocio = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', getNrorg());
                InstanceProvider::getService()->getParcnegocio()->createParcnegocio($nrparcnegocio, getNrorg(), $nmoperador, $nrinscricaoparc,
                    $cdtipoinscricao, getCurrentUserId(), $idativo);
                $entityManager->flush();
                return $nrparcnegocio;
            } else {
                return $parcnegocios[0]['NRPARCNEGOCIO'];
            }
        }
    }

    /**
     * @param $nrparcnegocio
     * @param $nmoperador
     * @param $nrorg
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $idativo
     * @param $parcnegocios
     * @param $oldCpf
     * @param $oldNmoperador
     * @param $entityManager
     * @return int|mixed|string
     * @throws Exception
     */
    private function setParcnegocioByEdit($nrparcnegocio, $nmoperador, $nrorg, $nrinscricaoparc, $cdtipoinscricao,
                                          $idativo, $parcnegocios, $oldCpf, $oldNmoperador, $entityManager)
    {

        if(!$nrparcnegocio) {
            if(empty($parcnegocios)) {
                $parcnegocioService = InstanceProvider::getService()->getParcnegocio();
                $nrparcnegocio = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $nrorg);
                $parcnegocioService->createParcnegocio($nrparcnegocio, $nrorg, $nmoperador, $nrinscricaoparc, $cdtipoinscricao, getCurrentUserId(), $idativo);
                $entityManager->flush();
            } else {
                $nrparcnegocio = $nrparcnegocio[0]['NRPARCNEGOCIO'];
            }

            $parcnegocioService->createParceria($nrparcnegocio, $nrorg, getCurrentUserId(), 'OPERADOR');
            $entityManager->flush();
            return $nrparcnegocio;
        } else if ($nrinscricaoparc != $oldCpf || $nmoperador != $oldNmoperador){
            $parcnegocioRepository = InstanceProvider::getRepository()->getParcnegocio();
            $parcnegocioRepository->updateParcnegocio($nrinscricaoparc, $cdtipoinscricao, $nmoperador, $nrorg, getCurrentUserId(), $nrparcnegocio);
            $entityManager->flush();
        }
        return $nrparcnegocio;
    }

    /**
     * @return string
     * @throws \Doctrine\DBAL\Exception
     */
    private function generateCdoperador(): string
    {
        $newCode = InstanceProvider::getNewCode()->getCodeByTableName('OPERADOR'.getNrorg(), getNrorg(), 1, 10, true);

        if(strlen(getNrorg() . $newCode) >= 12){
            $newCode = substr($newCode, strlen(getNrorg()));
        }
        $cdoperador = getNrorg() . $newCode;
        $cdoperador = str_pad($cdoperador, 12, 0, STR_PAD_LEFT);
        $operadorEntity = InstanceProvider::getRepository()->getOperador()->findBy(array('CDOPERADOR' => $cdoperador));
        do{
            $cdoperador +=   1;
            $cdoperador = str_pad($cdoperador, 12, 0, STR_PAD_LEFT);
            $operadorEntity = InstanceProvider::getRepository()->getOperador()->findBy(array('CDOPERADOR' => $cdoperador));
        }
        while(!empty($operadorEntity));
        return $cdoperador;
    }

    private function setChangeClient($cdcliente, $nrparcnegocio, $entityManager)
    {
        $relacionaparcRepository = InstanceProvider::getRepository()->getRelacionaparc();
        $inteclieRepository = InstanceProvider::getRepository()->getInteclie();
        $parceriaRepository = InstanceProvider::getRepository()->getParceria();

        if($cdcliente == null) {
            $relacionaparcRepository->updateIdativoInRelacionaparc(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();
            $inteclieRepository->updateIdativoInteclie(getCurrentUserId(), getNrorg(), $nrparcnegocio);
            $entityManager->flush();
            $parceriaRepository->updateIdativoInParceria(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();
        } else {
            if($parceriaRepository->hasParceriaTypeInterlocutor(getNrorg(), $nrparcnegocio) == 0){
                $nrparceria = InstanceProvider::getNewCode()->getCodeByTableName('PARCERIA', getNrorg());
                $parceriaRepository->createParceria($nrparcnegocio, getNrorg(), getCurrentUserId(), 'INTERLOCUTOR', $nrparceria);
                $entityManager->flush();
            } else {
                $parceriaRepository->updateIdativoInParceria(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::ACTIVE_ID);
                $entityManager->flush();
            }

            $relacionaparcRepository->updateIdativoInRelacionaparc(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();

            if($relacionaparcRepository->hasRelacionaparc(getNrorg(), $nrparcnegocio, $cdcliente) > 0){
                $inteclieRepository->activateInteclie(getNrorg(), $nrparcnegocio, $cdcliente, getCurrentUserId(), self::ACTIVE_ID);
                $entityManager->flush();
            }
        }
    }
}
