<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Parcnegocio as ParcnegocioRepository;
use ACL\Traits\TCrud;
use Exception;

class Parcnegocio {

    use TCrud;

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var ParcnegocioRepository
     */
    private $parcnegocioRepository;

    public function __construct(ParcnegocioRepository $parcnegocioRepository)
    {
        $this->parcnegocioRepository = $parcnegocioRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @return array
     */
    public function findAllParcnegocio(): array
    {
        return $this->parcnegocioRepository->findAllParcnegocio(getNrorg());
    }

    /**
     * @param $nmoperador
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $cdoperador
     * @return array
     * @throws \Doctrine\DBAL\ConnectionException
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function setParcnegocio($nmoperador, $nrinscricaoparc, $cdtipoinscricao, $cdoperador)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $organizationId = $this->libEnvironment->getCurrentOrganizationId();
            $cdoperinclusao = $this->libEnvironment->getCdOperador();

            /**
             * Seta os valores padrão e cria o registro na tabela PARCNEGOCIO
             */
            $idativo = 'S';
            $cdtipoparcprincipal = 'OPERADOR';
            $idpessoafisica = 'S';
            $idinstituicao = 'N';
            $idparcfundido = 'N';

            $nrparcnegocio = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $organizationId, 1, 10, true);

            $this->parcnegocioRepository->createParcnegocio($nrparcnegocio, $organizationId, $nmoperador, $nrinscricaoparc,
                                                            $cdtipoinscricao, $cdoperinclusao, $idativo, $cdtipoparcprincipal,
                                                            $idpessoafisica, $idinstituicao, $idparcfundido);
            $entityManager->flush();

            /**
             * Seta os valore(s) padrão e cria o registro na tabela PARCERIA
             */
            $cdtipoparceiro = 'OPERADOR';
            $nrparceria = InstanceProvider::getNewCode()->getCodeByTableName('PARCERIA', $organizationId);
            $this->parcnegocioRepository->createParceria($nrparcnegocio, $organizationId, $cdoperinclusao,
                                                        $cdtipoparceiro, $nrparceria);
            $entityManager->flush();

            /**
             * Recupera o Operador e seta o valor do NRPARCNEGOCIO
             * que acabou de ser criado na tabela de OPERADOR
             */
            $operador = $this->findOne('OPERADOR', '*', "WHERE CDOPERADOR = '{$cdoperador}'", $organizationId);
            $fieldsValues = array(
                'NRPARCNEGOCIO'  => $nrparcnegocio,
            );
            $conditions = "CDOPERADOR = '{$operador['CDOPERADOR']}'";
            $this->update('OPERADOR', $fieldsValues, $conditions);
            $entityManager->flush();

            $connection->commit();

            return array(
                'NRPARCNEGOCIO'   => $nrparcnegocio,
                'NMPRINCIPALPARC' => $nmoperador
            );
        } catch (Exception $e) {
            $connection->rollBack();
            throw $e;
        }


    }

    /**
     * @param $nrparcnegocio
     * @param $organizationId
     * @param $nmoperador
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $cdoperinclusao
     * @param $idativo
     * @param string $cdtipoparcprincipal
     * @param string $idpessoafisica
     * @param string $idinstituicao
     * @param string $idparcfundido
     * @throws Exception
     */
    public function createParcnegocio($nrparcnegocio, $organizationId, $nmoperador, $nrinscricaoparc, $cdtipoinscricao,
                                      $cdoperinclusao, $idativo, $cdtipoparcprincipal = 'OPERADOR', $idpessoafisica = 'S',
                                      $idinstituicao = 'N', $idparcfundido = 'N')
    {

        $this->parcnegocioRepository->createParcnegocio($nrparcnegocio, $organizationId, $nmoperador, $nrinscricaoparc,
                                                        $cdtipoinscricao, $cdoperinclusao, $idativo, $cdtipoparcprincipal,
                                                        $idpessoafisica, $idinstituicao, $idparcfundido);
    }

    /**
     * @param $nrparcnegocio
     * @param $nrorg
     * @param $cdoperadorInclusao
     * @param $cdtipoparceiro
     * @throws Exception
     */
    public function createParceria($nrparcnegocio, $nrorg, $cdoperadorInclusao, $cdtipoparceiro)
    {
        $nrparceria = InstanceProvider::getNewCode()->getCodeByTableName('PARCERIA', $nrorg);
        $this->parcnegocioRepository->createParceria($nrparcnegocio, $nrorg, $cdoperadorInclusao, $cdtipoparceiro, $nrparceria);
    }

}
